/*
 * ====================================================================
 *
 * MIT License
 * 
 * Copyright (c) [2022] 
 * [Jan Grossniklaus, Jan Kunert, Sascha Hildebrands, Kevin Hinrichs]
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy, 
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * urnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall
 * be included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * 
 * ====================================================================
 */

/*
 + --------------------------------------------------------------------
 + 
 + Created    :  13/03/2022
 + Owner      :  Kevin Hinrichs
 +
 + Description:  Hardware setup for Texas Instruments EK-TM4C123GXL
 +               TM4C123G LaunchPad Evaluation Kit.
 +
 + --------------------------------------------------------------------
 */

#include <stdint.h>
#include <stdbool.h>
#include <stdio.h>
#include <arm_math.h>

#include "project.h" // global header file
#include "tm4c123gxl_init.h"

#include "inc/hw_ints.h"
#include "inc/hw_memmap.h"

#include "driverlib/sysctl.h"
#include "driverlib/gpio.h"
#include "driverlib/pin_map.h"
#include "driverlib/rom.h"
#include "driverlib/rom_map.h"
#include "driverlib/interrupt.h"
#include "driverlib/timer.h"
#include "driverlib/udma.h"

//*********************************************************************
//
// The error routine that is called if the driver library error.
//
//*********************************************************************
#ifdef DEBUG
void
__error__(char *pcFilename, uint32_t ui32Line)
{
}
#endif

//*********************************************************************
//
// Interrupts Handler
//
//*********************************************************************
void uDMAErrorIntHandler(void)
{
    #ifndef DEBUG_OFF
    MAP_uDMAErrorStatusClear();
    uDMAErrorHandlerCounter++;
    #endif
}

//*********************************************************************
//
// Functions
//
//*********************************************************************
void init_SYS_CLOCK()
{
    // 80Mhz Clock
    MAP_SysCtlClockSet(SYSCTL_SYSDIV_2_5 | SYSCTL_USE_PLL | 
                       SYSCTL_OSC_MAIN | SYSCTL_XTAL_16MHZ);
    
    MAP_SysCtlDelay(10);
}

void init_FPU()
{
    MAP_FPUEnable();
    MAP_FPULazyStackingEnable();
    MAP_SysCtlDelay(10);
}

void init_GPIO_CLOCK_A()
{
    MAP_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOA);
    while(!MAP_SysCtlPeripheralReady(SYSCTL_PERIPH_GPIOA));
}

void init_GPIO_CLOCK_B()
{
    MAP_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOB);
    while(!MAP_SysCtlPeripheralReady(SYSCTL_PERIPH_GPIOB));
}

void init_GPIO_CLOCK_C()
{
    MAP_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOC);
    while(!MAP_SysCtlPeripheralReady(SYSCTL_PERIPH_GPIOC));
}

void init_GPIO_CLOCK_D()
{
    MAP_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOD);
    while(!MAP_SysCtlPeripheralReady(SYSCTL_PERIPH_GPIOD));
}

void init_GPIO_CLOCK_E()
{
    MAP_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOE);
    while(!MAP_SysCtlPeripheralReady(SYSCTL_PERIPH_GPIOE));
}

void init_GPIO_CLOCK_F()
{
    MAP_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOF);
    while(!MAP_SysCtlPeripheralReady(SYSCTL_PERIPH_GPIOF));
}

void init_uDMA_TRIGGER_0()
{
    // GPIO TIGGER 0: PA7
    MAP_GPIODirModeSet(GPIO_PORTA_BASE, GPIO_PIN_7, 
                       GPIO_DIR_MODE_IN);
    
    MAP_GPIOPadConfigSet(GPIO_PORTA_BASE, GPIO_PIN_7, 
                         GPIO_STRENGTH_2MA, GPIO_PIN_TYPE_STD_WPU);
    
    MAP_GPIODMATriggerEnable(GPIO_PORTA_BASE, GPIO_PIN_7);
    
    MAP_GPIOIntTypeSet(GPIO_PORTA_BASE, GPIO_PIN_7, GPIO_FALLING_EDGE);
    MAP_GPIOIntEnable(GPIO_PORTA_BASE, GPIO_INT_DMA);
    
    // ------------------ //
    // Priority Levels
    // Priority 4 = 0xE0 ->lower
    // Priority 3 = 0x60
    // Priority 2 = 0x20
    // Priority 1 = 0x00 ->higher
    // ------------------ //
    
    MAP_IntPrioritySet(INT_GPIOA, 0x00); // Priority 1 !!!
    MAP_IntEnable(INT_GPIOA);
}

void init_uDMA_TRIGGER_1()
{
    // GPIO TIGGER C: PC7
    MAP_GPIODirModeSet(GPIO_PORTC_BASE, GPIO_PIN_7, 
                       GPIO_DIR_MODE_IN);
    
    MAP_GPIOPadConfigSet(GPIO_PORTC_BASE, GPIO_PIN_7, 
                         GPIO_STRENGTH_2MA, GPIO_PIN_TYPE_STD_WPU);
    
    MAP_GPIODMATriggerEnable(GPIO_PORTC_BASE, GPIO_PIN_7);
    
    MAP_GPIOIntTypeSet(GPIO_PORTC_BASE, GPIO_PIN_7, GPIO_FALLING_EDGE);
    MAP_GPIOIntEnable(GPIO_PORTC_BASE, GPIO_INT_DMA);
   
    // ------------------ //
    // Priority Levels
    // Priority 4 = 0xE0 ->lower
    // Priority 3 = 0x60
    // Priority 2 = 0x20
    // Priority 1 = 0x00 ->higher
    // ------------------ //
    
    MAP_IntPrioritySet(INT_GPIOC, 0x00); // Priority 1 !!!
    MAP_IntEnable(INT_GPIOC);
}

void init_uDMA_TRIGGER_2()
{
    // GPIO TIGGER D: PD3
    MAP_GPIODirModeSet(GPIO_PORTD_BASE, GPIO_PIN_3, 
                       GPIO_DIR_MODE_IN);
    
    MAP_GPIOPadConfigSet(GPIO_PORTD_BASE, GPIO_PIN_3, 
                         GPIO_STRENGTH_2MA, GPIO_PIN_TYPE_STD_WPU);
    
    MAP_GPIODMATriggerEnable(GPIO_PORTD_BASE, GPIO_PIN_3);
    
    MAP_GPIOIntTypeSet(GPIO_PORTD_BASE, GPIO_PIN_3, GPIO_FALLING_EDGE);
    MAP_GPIOIntEnable(GPIO_PORTD_BASE, GPIO_INT_DMA);
    
    // ------------------ //
    // Priority Levels
    // Priority 4 = 0xE0 ->lower
    // Priority 3 = 0x60
    // Priority 2 = 0x20
    // Priority 1 = 0x00 ->higher
    // ------------------ //
    
    MAP_IntPrioritySet(INT_GPIOD, 0x00); // Priority 1 !!!
    MAP_IntEnable(INT_GPIOD);
}

void init_uDMA_TRIGGER_3()
{
    // GPIO TIGGER E: PF2
    MAP_GPIODirModeSet(GPIO_PORTF_BASE, GPIO_PIN_2, 
                       GPIO_DIR_MODE_IN);
    
    MAP_GPIOPadConfigSet(GPIO_PORTF_BASE, GPIO_PIN_2, 
                         GPIO_STRENGTH_2MA, GPIO_PIN_TYPE_STD_WPU);
    
    MAP_GPIODMATriggerEnable(GPIO_PORTF_BASE, GPIO_PIN_2);
    
    MAP_GPIOIntTypeSet(GPIO_PORTF_BASE, GPIO_PIN_2, GPIO_FALLING_EDGE);
    MAP_GPIOIntEnable(GPIO_PORTF_BASE, GPIO_INT_DMA);
    
    // ------------------ //
    // Priority Levels
    // Priority 4 = 0xE0 ->lower
    // Priority 3 = 0x60
    // Priority 2 = 0x20
    // Priority 1 = 0x00 ->higher
    // ------------------ //
    
    MAP_IntPrioritySet(INT_GPIOF, 0x00); // Priority 1 !!!
    MAP_IntEnable(INT_GPIOF);
}

void init_TIMER_0()
{
    MAP_SysCtlPeripheralEnable(SYSCTL_PERIPH_TIMER0);
    while(!MAP_SysCtlPeripheralReady(SYSCTL_PERIPH_TIMER0));
    
    // Timer 0A is the data signal 1: PB6
    // Timer 0B is the udma trigger signal to GPIO: PB7
    
    MAP_GPIOPinConfigure(GPIO_PB6_T0CCP0);
    MAP_GPIOPinConfigure(GPIO_PB7_T0CCP1);
    
    MAP_GPIOPinTypeTimer(GPIO_PORTB_BASE, GPIO_PIN_6);
    MAP_GPIOPinTypeTimer(GPIO_PORTB_BASE, GPIO_PIN_7);
    
    MAP_TimerConfigure(TIMER0_BASE, TIMER_CFG_SPLIT_PAIR | 
                       TIMER_CFG_A_PWM | TIMER_CFG_B_PWM);
    
    MAP_TimerLoadSet(TIMER0_BASE, TIMER_BOTH, 
                     (SysCtlClockGet()/SIGNAL_FREQUENZ) - 1);
    
    // start led signal with the reset signal
    MAP_TimerMatchSet(TIMER0_BASE, TIMER_A, SIGNAL_RESET);
    
    // DMA Request
    MAP_TimerMatchSet(TIMER0_BASE, TIMER_B, SIGNAL_DMA);
}

void init_TIMER_1()
{
    MAP_SysCtlPeripheralEnable(SYSCTL_PERIPH_TIMER1);
    while(!MAP_SysCtlPeripheralReady(SYSCTL_PERIPH_TIMER1));
    
    // Timer 1A is the data signal 2: PB4
    // Timer 1B is the udma trigger signal to GPIO: PB5
    
    MAP_GPIOPinConfigure(GPIO_PB4_T1CCP0);
    MAP_GPIOPinConfigure(GPIO_PB5_T1CCP1);
    
    MAP_GPIOPinTypeTimer(GPIO_PORTB_BASE, GPIO_PIN_4);
    MAP_GPIOPinTypeTimer(GPIO_PORTB_BASE, GPIO_PIN_5);
    
    MAP_TimerConfigure(TIMER1_BASE, TIMER_CFG_SPLIT_PAIR | 
                       TIMER_CFG_A_PWM | TIMER_CFG_B_PWM);
    
    MAP_TimerLoadSet(TIMER1_BASE, TIMER_BOTH, 
                     (SysCtlClockGet()/SIGNAL_FREQUENZ) - 1);
    
    // start led signal with the reset signal
    MAP_TimerMatchSet(TIMER1_BASE, TIMER_A, SIGNAL_RESET);
    
    // DMA Request
    MAP_TimerMatchSet(TIMER1_BASE, TIMER_B, SIGNAL_DMA);
}

void init_TIMER_2()
{
    MAP_SysCtlPeripheralEnable(SYSCTL_PERIPH_TIMER2);
    while(!MAP_SysCtlPeripheralReady(SYSCTL_PERIPH_TIMER2));
    
    // Timer 2A is the data signal 3: PB0
    // Timer 2B is the udma trigger signal to GPIO: PB1
    
    MAP_GPIOPinConfigure(GPIO_PB0_T2CCP0);
    MAP_GPIOPinConfigure(GPIO_PB1_T2CCP1);
    
    MAP_GPIOPinTypeTimer(GPIO_PORTB_BASE, GPIO_PIN_0);
    MAP_GPIOPinTypeTimer(GPIO_PORTB_BASE, GPIO_PIN_1);
    
    MAP_TimerConfigure(TIMER2_BASE, TIMER_CFG_SPLIT_PAIR | 
                       TIMER_CFG_A_PWM | TIMER_CFG_B_PWM);
    
    MAP_TimerLoadSet(TIMER2_BASE, TIMER_BOTH,
                     (SysCtlClockGet()/SIGNAL_FREQUENZ) - 1);
    
    // start led signal with the reset signal
    MAP_TimerMatchSet(TIMER2_BASE, TIMER_A, SIGNAL_RESET);
    
    // DMA Request
    MAP_TimerMatchSet(TIMER2_BASE, TIMER_B, SIGNAL_DMA);
}

void init_TIMER_3()
{
    MAP_SysCtlPeripheralEnable(SYSCTL_PERIPH_TIMER3);
    while(!MAP_SysCtlPeripheralReady(SYSCTL_PERIPH_TIMER3));
    
    // Timer 3A is the data signal 4: PB2
    // Timer 3B is the udma trigger signal to GPIO: PB3
    
    // special setup because GPIO is locked and preconfigured
    MAP_GPIOUnlockPin(GPIO_PORTB_BASE, GPIO_PIN_2);
    MAP_GPIOUnlockPin(GPIO_PORTB_BASE, GPIO_PIN_3);
    
    MAP_GPIOPinConfigure(GPIO_PB2_T3CCP0);
    MAP_GPIOPinConfigure(GPIO_PB3_T3CCP1);
    
    MAP_GPIOPinTypeTimer(GPIO_PORTB_BASE, GPIO_PIN_2);
    MAP_GPIOPinTypeTimer(GPIO_PORTB_BASE, GPIO_PIN_3);
    
    MAP_TimerConfigure(TIMER3_BASE, TIMER_CFG_SPLIT_PAIR | 
                       TIMER_CFG_A_PWM | TIMER_CFG_B_PWM);
    
    MAP_TimerLoadSet(TIMER3_BASE, TIMER_BOTH, 
                     (SysCtlClockGet()/SIGNAL_FREQUENZ) - 1);
    
    // start led signal with the reset signal
    MAP_TimerMatchSet(TIMER3_BASE, TIMER_A, SIGNAL_RESET);
    
    // DMA Request
    MAP_TimerMatchSet(TIMER3_BASE, TIMER_B, SIGNAL_DMA);
}

void init_WTIMER_5()
{
    MAP_SysCtlPeripheralEnable(SYSCTL_PERIPH_WTIMER5);
    while(!MAP_SysCtlPeripheralReady(SYSCTL_PERIPH_WTIMER5));
    
    // WTimer 5A is PD6 and the trigger for adc
    // WTimer 5B is PD7 and the timer for led reset time
    MAP_TimerConfigure(WTIMER5_BASE, TIMER_CFG_SPLIT_PAIR | 
                       TIMER_CFG_A_PERIODIC | TIMER_CFG_A_ONE_SHOT);
    
    // WTimer 5B
    // set ADC sampling frequency
    MAP_TimerLoadSet(WTIMER5_BASE, TIMER_B, 
                     (SysCtlClockGet()/SIGNAL_RESET_FREQUENZ) - 1);
    
    // WTimer 5B
    // enable timer done interrupt
    TimerIntEnable(WTIMER5_BASE, TIMER_TIMB_TIMEOUT);
    IntEnable(INT_WTIMER5B);
}

void init_uDMA()
{
    MAP_SysCtlPeripheralEnable(SYSCTL_PERIPH_UDMA);
    while(!MAP_SysCtlPeripheralReady(SYSCTL_PERIPH_UDMA));
    
    // general uDMA setup
    MAP_uDMAEnable();
    MAP_SysCtlDelay(10);
    
    // declare controltable
    MAP_uDMAControlBaseSet(uDMAControlTable);
    MAP_SysCtlDelay(10);
    
    #ifndef DEBUG_OFF
    MAP_IntEnable(INT_UDMAERR);
    #endif
}

void init_uDMA_LED0()
{
    // signal 0
    // uDMA Channel 4
    MAP_uDMAChannelAssign(UDMA_CH4_GPIOA);
    MAP_uDMAChannelAttributeDisable(4, 
                                    UDMA_ATTR_ALTSELECT | 
                                    UDMA_ATTR_HIGH_PRIORITY | 
                                    UDMA_ATTR_REQMASK | 
                                    UDMA_ATTR_USEBURST);
    
    MAP_uDMAChannelControlSet(4 | UDMA_PRI_SELECT, 
                              UDMA_SIZE_16 | UDMA_SRC_INC_16 | 
                              UDMA_DST_INC_NONE | UDMA_ARB_1);
    
    MAP_uDMAChannelControlSet(4 | UDMA_ALT_SELECT, 
                              UDMA_SIZE_16 | UDMA_SRC_INC_16 | 
                              UDMA_DST_INC_NONE | UDMA_ARB_1);
    
    //MAP_uDMAChannelAttributeEnable(4, UDMA_ATTR_HIGH_PRIORITY);
}

void init_uDMA_LED1()
{
    // signal 1
    // uDMA Channel 6
    MAP_uDMAChannelAssign(UDMA_CH6_GPIOC);
    MAP_uDMAChannelAttributeDisable(6, 
                                    UDMA_ATTR_ALTSELECT | 
                                    UDMA_ATTR_HIGH_PRIORITY | 
                                    UDMA_ATTR_REQMASK | 
                                    UDMA_ATTR_USEBURST);
    
    MAP_uDMAChannelControlSet(6 | UDMA_PRI_SELECT, 
                              UDMA_SIZE_16 | UDMA_SRC_INC_16 | 
                              UDMA_DST_INC_NONE | UDMA_ARB_1);
    
    MAP_uDMAChannelControlSet(6 | UDMA_ALT_SELECT, 
                              UDMA_SIZE_16 | UDMA_SRC_INC_16 | 
                              UDMA_DST_INC_NONE | UDMA_ARB_1);
    
    //MAP_uDMAChannelAttributeEnable(6, UDMA_ATTR_HIGH_PRIORITY);
}

void init_uDMA_LED2()
{
    // signal 2
    // uDMA Channel 7
    MAP_uDMAChannelAssign(UDMA_CH7_GPIOD);
    MAP_uDMAChannelAttributeDisable(7, 
                                    UDMA_ATTR_ALTSELECT | 
                                    UDMA_ATTR_HIGH_PRIORITY | 
                                    UDMA_ATTR_REQMASK | 
                                    UDMA_ATTR_USEBURST);
    
    MAP_uDMAChannelControlSet(7 | UDMA_PRI_SELECT, 
                              UDMA_SIZE_16 | UDMA_SRC_INC_16 | 
                              UDMA_DST_INC_NONE | UDMA_ARB_1);
    
    MAP_uDMAChannelControlSet(7 | UDMA_ALT_SELECT, 
                              UDMA_SIZE_16 | UDMA_SRC_INC_16 | 
                              UDMA_DST_INC_NONE | UDMA_ARB_1);
    
    //MAP_uDMAChannelAttributeEnable(7, UDMA_ATTR_HIGH_PRIORITY);
}

void init_uDMA_LED3()
{
    // signal 3
    // uDMA Channel 15
    MAP_uDMAChannelAssign(UDMA_CH15_GPIOF);
    MAP_uDMAChannelAttributeDisable(15, 
                                    UDMA_ATTR_ALTSELECT | 
                                    UDMA_ATTR_HIGH_PRIORITY | 
                                    UDMA_ATTR_REQMASK | 
                                    UDMA_ATTR_USEBURST);
    
    MAP_uDMAChannelControlSet(15 | UDMA_PRI_SELECT, 
                              UDMA_SIZE_16 | UDMA_SRC_INC_16 | 
                              UDMA_DST_INC_NONE | UDMA_ARB_1);
    
    MAP_uDMAChannelControlSet(15 | UDMA_ALT_SELECT, 
                              UDMA_SIZE_16 | UDMA_SRC_INC_16 | 
                              UDMA_DST_INC_NONE | UDMA_ARB_1);
    
    //MAP_uDMAChannelAttributeEnable(15, UDMA_ATTR_HIGH_PRIORITY);
}

void init_GPIO_DEBUG()
{
    // Port A
    //MAP_GPIOUnlockPin(GPIO_PORTA_BASE, GPIO_PIN_2);
    
    MAP_GPIODirModeSet(GPIO_PORTA_BASE, GPIO_PIN_2 | GPIO_PIN_3 | 
                       GPIO_PIN_4 | GPIO_PIN_5 | GPIO_PIN_6, 
                       GPIO_DIR_MODE_OUT);
    
    MAP_GPIOPadConfigSet(GPIO_PORTA_BASE, GPIO_PIN_2 | GPIO_PIN_3 | 
                         GPIO_PIN_4 | GPIO_PIN_5 | GPIO_PIN_6, 
                         GPIO_STRENGTH_4MA, GPIO_PIN_TYPE_STD);
    //-----------------------------------------------------------------
    
    // Port C
    MAP_GPIODirModeSet(GPIO_PORTC_BASE, GPIO_PIN_4 | GPIO_PIN_5 | 
                       GPIO_PIN_6, GPIO_DIR_MODE_OUT);
    
    MAP_GPIOPadConfigSet(GPIO_PORTC_BASE, GPIO_PIN_4 | GPIO_PIN_5 | 
                         GPIO_PIN_6, GPIO_STRENGTH_4MA, 
                         GPIO_PIN_TYPE_STD);
    //-----------------------------------------------------------------
    
    // Port D
    // in used for adc trigger for poti
    /*
    MAP_GPIODirModeSet(GPIO_PORTD_BASE, GPIO_PIN_7, 
                       GPIO_DIR_MODE_OUT);
    
    MAP_GPIOPadConfigSet(GPIO_PORTD_BASE, GPIO_PIN_7, 
                         GPIO_STRENGTH_4MA, GPIO_PIN_TYPE_STD);
    */
    //-----------------------------------------------------------------
    
    // Port E
    MAP_GPIODirModeSet(GPIO_PORTE_BASE, GPIO_PIN_0 | GPIO_PIN_1 | 
                       GPIO_PIN_2 | GPIO_PIN_3 | GPIO_PIN_4 | GPIO_PIN_5, 
                       GPIO_DIR_MODE_OUT);
    
    MAP_GPIOPadConfigSet(GPIO_PORTE_BASE, GPIO_PIN_0 | GPIO_PIN_1 | 
                       GPIO_PIN_2 | GPIO_PIN_3 | GPIO_PIN_4 | GPIO_PIN_5, 
                       GPIO_STRENGTH_4MA, GPIO_PIN_TYPE_STD);
    //-----------------------------------------------------------------
    
    // Port F
    MAP_GPIOUnlockPin(GPIO_PORTF_BASE, GPIO_PIN_0);
    
    MAP_GPIODirModeSet(GPIO_PORTF_BASE, GPIO_PIN_0 | GPIO_PIN_1 | 
                       GPIO_PIN_3 | GPIO_PIN_4, GPIO_DIR_MODE_OUT);
    
    MAP_GPIOPadConfigSet(GPIO_PORTF_BASE, GPIO_PIN_0 | GPIO_PIN_1 | 
                         GPIO_PIN_3 | GPIO_PIN_4, 
                         GPIO_STRENGTH_4MA, GPIO_PIN_TYPE_STD);
    //-----------------------------------------------------------------
}

void init_hardware()
{
    init_SYS_CLOCK();
    init_FPU();
    
    init_GPIO_CLOCK_A();
    init_GPIO_CLOCK_B();
    init_GPIO_CLOCK_C();
    init_GPIO_CLOCK_D();
    init_GPIO_CLOCK_E();
    init_GPIO_CLOCK_F();
    
    init_uDMA_TRIGGER_0();
    init_uDMA_TRIGGER_1();
    init_uDMA_TRIGGER_2();
    init_uDMA_TRIGGER_3();
    
    init_TIMER_0();
    init_TIMER_1();
    init_TIMER_2();
    init_TIMER_3();
    
    init_WTIMER_5();
    
    init_uDMA();
    init_uDMA_LED0();
    init_uDMA_LED1();
    init_uDMA_LED2();
    init_uDMA_LED3();
}